@@ -27,14 +27,14 @@ pip install svcco
 '''
 from __future__ import annotations
 
-__version__ = "0.4.26"
+__version__ = "0.5.27"
 
 import traceback
 import warnings
 from pathlib import Path
 from typing import overload
 
-from svcco import implicit, collision, branch_addition, sv_interface
+from svcco import implicit, collision, branch_addition, sv_interface, forest_utils
 
 #@overload
 #def svcco(*modules: Path | str, output_directory: None = None,
@@ -44,6 +44,7 @@ from . import implicit
 from . import collision
 from . import branch_addition
 from . import sv_interface
+from . import forest_utils
 
 from .tree import tree, forest, perfusion_territory
 from .implicit.implicit import surface
