@@ -25,7 +25,7 @@ import random
 
 def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
                all_max_attempts=40,max_attemps=10,sampling=20,max_skip=8,
-               flow_ratio=None):
+               flow_ratio=None,radius_buffer=0,isforest=False):
     number_edges       = tree.parameters['edge_num']
     threshold = ((tree.boundary.volume)**(1/3)/(number_edges**threshold_exponent))
     mu                 = tree.parameters['mu']
@@ -53,6 +53,7 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
         start_rng = time.time()
         nonconvex_solve = True
         if len(tree.rng_points) == 0:
+            print('repick')
             rng_points,_ = tree.boundary.pick(size=len(tree.boundary.tet_verts),homogeneous=True,replacement=False)
             rng_points = rng_points.tolist()
         else:
@@ -70,6 +71,7 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
                 #point,_ = tree.boundary.pick(homogeneous=True)
                 #time1 += time.time() - start1
                 if len(rng_points) == 1:
+                    print('repick_low')
                     rng_points,_ = tree.boundary.pick(size=len(tree.boundary.tet_verts),homogeneous=True,replacement=False)
                     rng_points = rng_points.tolist()
                 point = np.array(rng_points.pop(0))
@@ -310,14 +312,14 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
             bif     = results[5][idx]
             start = time.time()
             no_collision = collision_free(tree.data,results,idx,terminal,
-                                          vessel,tree.radius_buffer)
+                                          vessel,radius_buffer)
             collision_time += time.time()-start
 
             if no_collision:
                 new_branch_found = True
                 start = time.time()
                 data,sub_division_map,sub_division_index = add_bifurcation(tree,vessel,terminal,
-                                                                           results,idx)
+                                                                           results,idx,isforest=isforest)
                 add_time += time.time()-start
                 total_time += time.time()-start_total
                 tree.time['search'].append(search_time)
