@@ -5,9 +5,11 @@ from scipy import interpolate
 def get_local_points(data,vessel,terminal,num,isclamped):
     proximal = data[vessel,0:3]
     distal   = data[vessel,3:6]
+    direction = distal - proximal
     if np.all(vessel == 0) and isclamped:
         length = data[vessel,20]
-        points = proximal + length*np.linspace(0,1,num)
+        shift = direction.reshape(-1,1)*length*np.linspace(0,1,num)
+        points = proximal.reshape(1,-1) + shift.T
     else:
         line   = np.linspace(0,1,num)
         s,t    = np.meshgrid(line,line)
@@ -16,6 +18,7 @@ def get_local_points(data,vessel,terminal,num,isclamped):
         points = proximal.reshape(1,-1)*(1-t)*s + \
                    distal.reshape(1,-1)*(t*s)+\
                    terminal.reshape(1,-1)*(1-s)
+        #print('SHAPE: {}'.format(points.shape))
     return points
 
 """
