@@ -737,9 +737,9 @@ def hessian(xyzk,KDTree=None,patch_points=None,b_coef=None,h=None,q=None,sol_mat
     dx22 = ((-dwx*np.tanh(fsg)+dx*weights)*total+(dwx_total*weights*np.tanh(fsg)))/total**2
     dy22 = ((-dwy*np.tanh(fsg)+dy*weights)*total+(dwy_total*weights*np.tanh(fsg)))/total**2
     dz22 = ((-dwz*np.tanh(fsg)+dz*weights)*total+(dwz_total*weights*np.tanh(fsg)))/total**2
-    print('dx: {}'.format(np.sum(dx22)))
-    print('dy: {}'.format(np.sum(dy22)))
-    print('dz: {}'.format(np.sum(dz22)))
+    #print('dx: {}'.format(np.sum(dx22)))
+    #print('dy: {}'.format(np.sum(dy22)))
+    #print('dz: {}'.format(np.sum(dz22)))
     #dx2 = ((-dwx*np.tanh(fsg)+dx*weights)*total+(dwx_total*weights*np.tanh(fsg)))/total**2
     #dy2 = ((-dwy*np.tanh(fsg)+dy*weights)*total+(dwy_total*weights*np.tanh(fsg)))/total**2
     #dz2 = ((-dwz*np.tanh(fsg)+dz*weights)*total+(dwz_total*weights*np.tanh(fsg)))/total**2
@@ -1088,7 +1088,7 @@ def linesearch_d2(t,point=None,normal=None,patch_points=None,b_coef=None,h=None,
            2.0*(nx*dxc + ny*dyc + nz*dzc)**2*np.tanh(rc**0.5)/rc**1.0))/(h + rc**0.5)**q
 
 
-    part = np.sum(np.nan_to_num(A*r**1.5 + 3.0*B1*dx*r**0.5 + 3.0*B2*dy*r**0.5 + 
+    part = np.sum(np.nan_to_num(A*r**1.5 + 3.0*B1*dx*r**0.5 + 3.0*B2*dy*r**0.5 +
                   3.0*B3*dz*r**0.5 + C1*(nx*t + x) + C2*(ny*t + y) +
                   C3*(nz*t + z) + C4),axis=1)
 
