@@ -309,6 +309,16 @@ class surface:
                 solution_matrix_tmp[point,3] = patch.a[patch.ndim*3+point]
             solution_matrix.append(solution_matrix_tmp)
             b_sol.append(b_sol_tmp)
+        self.solution_matrix = solution_matrix
+        self.a_coef          = a_coef
+        self.b_coef          = b_coef
+        self.b_sol           = b_sol
+        self.n_points        = n_points
+        self.patch_points    = patch_points
+        self.patch_x         = patch_x
+        self.patch_y         = patch_y
+        self.patch_z         = patch_z
+        self.patch_points_m  = patch_points_m
         if len(self.patches) > 1:
             functions = []
             KDTree = spatial.KDTree(np.array([patch_x,patch_y,patch_z]).T)
@@ -390,6 +400,13 @@ class surface:
         self.DD = DD
         self.CDD = CDD
         """
+    def build_individual(self,idx,h=0,q=1):
+        func_marching = partial(function_marching,patch_points=[self.patch_points_m[idx]],
+                                a_coef=[self.a_coef[idx]],b_coef=[self.b_coef[idx]],h=h,q=q,
+                                patch_x=np.array([self.patch_x[idx]]),patch_y=np.array([self.patch_y[idx]]),
+                                patch_z=np.array([self.patch_z[idx]]))
+        return func_marching
+        
     def within(self,x,y,z,k):
         return self.DD[0]([x,y,z,k]) < 0
 
