@@ -1,6 +1,7 @@
 import numpy as np
 import vtk
 from vtk.util import numpy_support
+import pyvista as pv
 
 def load3d(filename,subdivisions=0):
     vtk_reader = {'stl':vtk.vtkSTLReader,
@@ -50,28 +51,36 @@ def load3d(filename,subdivisions=0):
             normals = normal_point_data.GetNormals()
             normals = numpy_support.vtk_to_numpy(normals)
         else:
-            normals = numpy_support.vtk_to_numpy(normals)
-        if normals is None:
-            normals_gen = vtk.vtkPolyDataNormals()
-            normals_gen.SplittingOn()
-            normals_gen.ComputeCellNormalsOff()
-            normals_gen.ComputePointNormalsOn()
-            normals_gen.SetInputData(data)
-            normals_gen.Update()
-            normal_polydata = normals_gen.GetOutput()
-            normal_point_data = normal_polydata.GetPointData()
-            normals = normal_point_data.GetNormals()
-            normals = numpy_support.vtk_to_numpy(normals).tolist()
-            points = []
-            norms = []
-            for idx in range(normal_polydata.GetNumberOfCells()):
-                tri = normal_polydata.GetCell(idx)
-                tri_points = numpy_support.vtk_to_numpy(tri.GetPoints().GetData()).tolist()
-                points.extend(tri_points)
-                for jdx in range(len(tri_points)):
-                    norms.append(normals[idx])
-            points = np.array(points)
-            normals = np.array(norms)
+            #normals = numpy_support.vtk_to_numpy(normals)
+            if normals is None:
+                """
+                normals_gen = vtk.vtkPolyDataNormals()
+                normals_gen.SplittingOn()
+                normals_gen.ComputeCellNormalsOff()
+                normals_gen.ComputePointNormalsOn()
+                normals_gen.SetInputData(data)
+                normals_gen.Update()
+                normal_polydata = normals_gen.GetOutput()
+                normal_point_data = normal_polydata.GetPointData()
+                normals = normal_point_data.GetNormals()
+                normals = numpy_support.vtk_to_numpy(normals).tolist()
+                points = []
+                norms = []
+                for idx in range(normal_polydata.GetNumberOfCells()):
+                    tri = normal_polydata.GetCell(idx)
+                    tri_points = numpy_support.vtk_to_numpy(tri.GetPoints().GetData()).tolist()
+                    points.extend(tri_points)
+                    for jdx in range(len(tri_points)):
+                        norms.append(normals[idx])
+                """
+                obj = pv.PolyData(var_inp=data)
+                obj = obj.compute_normals()
+                #points = np.array(points)
+                #normals = np.array(norms)
+                points = obj.points
+                normals = obj['Normals']
+            else:
+                normals = numpy_support.vtk_to_numpy(normals)
         upt,uid = np.unique(points,axis=0,return_index=True)
         points = points[uid]
         normals = normals[uid]
