@@ -12,7 +12,6 @@ from time import time
 from tqdm import tqdm
 from pickle import loads
 #import pandas as pd
-import plotly.graph_objects as go
 import vtk
 
 from ..tests.bumpy_sphere import bumpy_sphere
@@ -137,7 +136,22 @@ def plot_volume(mpu_object,resolution=20,workers=1,cmin=-1,cmax=0,surface_count=
                     surface_count=surface_count),go.Scatter3d(x = mpu_object.points[:,0],y=mpu_object.points[:,1],z=mpu_object.points[:,2])))
     fig.show()
 
-
+def plot_volume_individual(func,x_range,y_range,z_range,buf=1.25,res=20,surface_count=15,cmin=-1,cmax=0):
+    res = res*1j
+    x,y,z = np.ogrid[x_range[0]-buf:x_range[1]+buf:res,
+                     y_range[0]-buf:y_range[1]+buf:res,
+                     z_range[0]-buf:z_range[1]+buf:res]
+    results = func(x,y,z)
+    fig = go.Figure(data=(go.Isosurface(
+                    x=x.flatten(),
+                    y=y.flatten(),
+                    z=z.flatten(),
+                    value=results.flatten(),
+                    opacity=0.1,
+                    isomin=cmin,
+                    isomax=cmax,
+                    surface_count=surface_count)))
+    fig.show()
 
 def plot_slice_error(mpu_object,resolution=20,k=None,workers=1,plane_axis=2,plane_value=0,unit_scale='mm'):
     DIMS,results_exact = mpu_meshgrid(mpu_object,res=resolution,workers=workers,
