@@ -11,6 +11,9 @@ from .branch_addition.set_root import *
 from .branch_addition.add_edge import *
 from .branch_addition.add_bifurcation import *
 from .branch_addition.add_branches_v3 import *
+from .forest_utils.connect import *
+from .forest_utils.smooth import *
+from .forest_utils.compete_add import *
 from .sv_interface.get_sv_data import *
 from .sv_interface.options import *
 from .sv_interface.build_files import *
@@ -127,11 +130,12 @@ class tree:
         #self.sub_division_map = [-1]
         #self.sub_division_index = np.array([0])
 
-    def add(self,low,high,isforest=False):
+    def add(self,low,high,isforest=False,radius_buffer=0.01):
         vessel,data,sub_division_map,sub_division_index = add_branch(self,low,high,threshold_exponent=0.5,
                                                                      threshold_adjuster=0.75,all_max_attempts=40,
                                                                      max_attemps=3,sampling=20,max_skip=8,
-                                                                     flow_ratio=None)
+                                                                     flow_ratio=None,radius_buffer=radius_buffer,
+                                                                     isforest=isforest)
         if isforest:
             return vessel,data,sub_division_map,sub_division_index
         else:
@@ -143,8 +147,8 @@ class tree:
     def n_add(self,n):
         self.rng_points,_ = self.boundary.pick(size=40*n,homogeneous=True)
         self.rng_points = self.rng_points.tolist()
-        #for i in tqdm(range(n),desc='Adding vessels'):
-        for i in range(n):
+        for i in tqdm(range(n),desc='Adding vessels'):
+            #for i in range(n):
             #self.rng_points = self.rng_points.tolist()
             self.add(-1,0)
 
@@ -168,10 +172,34 @@ class tree:
         #plt.show()
 
     def show(self,surface=False,vessel_colors='red',background_color='white',
-             resolution=100,show_segments=True,save=False,name=None,show=True):
+             resolution=100,show_segments=True,save=False,name=None,show=True,
+             surface_color='red',other_surface_color='blue'):
         models = []
         actors = []
-        colors = vtk.vtkNamedColors()
+        if not isinstance(vessel_colors,str):
+            colors = vtk.vtkColor3d()
+            colors.Set(vessel_colors[0],vessel_colors[1],vessel_colors[2])
+        else:
+            colors = vtk.vtkNamedColors()
+            colors = colors.GetColor3d(vessel_colors)
+        if not isinstance(background_color,str):
+            backcolors = vtk.vtkColor3d()
+            backcolors.Set(background_color[0],background_color[1],background_color[2])
+        else:
+            backcolors = vtk.vtkNamedColors()
+            backcolors = backcolors.GetColor3d(background_color)
+        if not isinstance(surface_color,str):
+            surf_color = vtk.vtkColor3d()
+            surf_color.Set(surface_color[0],surface_color[1],surface_color[2])
+        else:
+            surf_color = vtk.vtkNamedColors()
+            surf_color = surf_color.GetColor3d(surface_color)
+        if not isinstance(other_surface_color,str):
+            other_surf_color = vtk.vtkColor3d()
+            other_surf_color.Set(other_surface_color[0],other_surface_color[1],other_surface_color[2])
+        else:
+            other_surf_color = vtk.vtkNamedColors()
+            other_surf_color = other_surf_color.GetColor3d(other_surface_color)
         if show_segments:
             if self.homogeneous:
                 data_subset = self.data[self.data[:,-1] > -1]
@@ -194,14 +222,14 @@ class tree:
                 actor  = vtk.vtkActor()
                 mapper.SetInputConnection(cyl.GetOutputPort())
                 actor.SetMapper(mapper)
-                actor.GetProperty().SetColor(colors.GetColor3d(vessel_colors))
+                actor.GetProperty().SetColor(colors)
                 actors.append(actor)
             if surface:
                 mapper = vtk.vtkPolyDataMapper()
                 actor  = vtk.vtkActor()
                 mapper.SetInputData(self.boundary.polydata)
                 actor.SetMapper(mapper)
-                actor.GetProperty().SetColor(colors.GetColor3d('red'))
+                actor.GetProperty().SetColor(surf_color)
                 actor.GetProperty().SetOpacity(0.25)
                 actors.append(actor)
                 for other_surface in self.boundary.subtracted_volumes:
@@ -209,11 +237,11 @@ class tree:
                     actor  = vtk.vtkActor()
                     mapper.SetInputData(other_surface.polydata)
                     actor.SetMapper(mapper)
-                    actor.GetProperty().SetColor(colors.GetColor3d('blue'))
+                    actor.GetProperty().SetColor(other_surf_color)
                     actor.GetProperty().SetOpacity(0.5)
                     actors.append(actor)
             renderer = vtk.vtkRenderer()
-            renderer.SetBackground(colors.GetColor3d(background_color))
+            renderer.SetBackground(backcolors)
 
             render_window = vtk.vtkRenderWindow()
             render_window.AddRenderer(renderer)
@@ -278,7 +306,14 @@ class tree:
         self.boundary = pickle.loads(file)
         file.close()
 
-    def export(self,steady=True,apply_distal_resistance=True,gui=True):
+    def export(self,steady=True,apply_distal_resistance=True,gui=True,cylinders=False):
+        if cylinders:
+            pv_data = []
+            models = self.show(show=False)
+            for m in models:
+                pv_data.append(pv.PolyData(var_inp=m.GetOutput()))
+            merge = pv_data[0].merge(pv_data[1:])
+            return merge
         interp_xyz,interp_r,interp_n,frames,branches = get_interpolated_sv_data(self.data)
         points,radii,normals    = sv_data(interp_xyz,interp_r,radius_buffer=self.radius_buffer)
         if steady:
@@ -296,19 +331,21 @@ class tree:
         options = file_options(time=time,flow=flow,gui=gui,distal_resistance=R)
         build(points,radii,normals,options)
 
-    def collision_free(self,outside_vessels):
-        return no_outside_collision(self,outside_vessels)
+    def collision_free(self,outside_vessels,radius_buffer=0.01):
+        return no_outside_collision(self,outside_vessels,radius_buffer=radius_buffer)
 
 class forest:
     def __init__(self,boundary=None,number_of_networks=1,
                  trees_per_network=[2],scale=None,start_points=None,
                  directions=None,root_lengths_high=None,
-                 root_lengths_low=None,convex=False):
+                 root_lengths_low=None,convex=False,compete=False):
         self.networks    = []
         self.connections = []
+        self.assignments = []
         self.backup      = []
         self.boundary    = boundary
         self.convex      = convex
+        self.compete     = compete
         self.number_of_networks = number_of_networks
         self.trees_per_network = trees_per_network
         if isinstance(start_points,type(None)):
@@ -325,6 +362,9 @@ class forest:
         self.root_lengths_low  = root_lengths_low
 
     def set_roots(self,scale=None,bounds=None):
+        if self.compete:
+            volume = deepcopy(self.boundary.volume)
+            self.boundary.volume = self.boundary.volume/self.number_of_networks
         if self.boundary is None:
             print("Need to assign boundary!")
             return
@@ -367,8 +407,12 @@ class forest:
         self.networks    = networks
         self.connections = connections
         self.backup      = backup
+        if self.compete:
+            self.boundary.volume = volume
 
-    def add(self,number_of_branches,network_id=0,radius_buffer=0.01):
+    def add(self,number_of_branches,network_id=0,radius_buffer=0.01,exact=True):
+        if self.compete:
+            network_id = -1
         if network_id == -1:
             exit_number = []
             active_networks = list(range(len(self.networks)))
@@ -377,54 +421,132 @@ class forest:
         else:
             exit_number = []
             active_networks = [network_id]
-            exit_number.append(self.networks[network_id][0].parameters['edge_num'] + number_of_branches)
-        while len(active_networks) > 0:
-            for nid in active_networks:
-                number_of_trees = len(self.networks[nid])
-                for njd in range(number_of_trees):
-                    success = False
-                    while not success:
-                        vessel,data,sub_division_map,sub_division_index = self.networks[nid][njd].add(-1,0,isforest=True)
-                        new_vessels = np.vstack((data[vessel,:],data[-2,:],data[-1,:]))
-                        repeat = False
-                        for nikd in range(len(self.networks)):
-                            if nikd == nid:
-                                check_trees = list(range(len(self.networks[nikd])))
-                                check_trees.remove(njd)
+            for nid in range(self.number_of_networks):
+                if nid in active_networks:
+                    exit_number.append(self.networks[nid][0].parameters['edge_num'] + number_of_branches)
+                else:
+                    exit_number.append(self.networks[nid][0].parameters['edge_num'])
+        for AN in active_networks:
+            for ATr in range(self.trees_per_network[AN]):
+                self.networks[AN][ATr].rng_points,_ = self.boundary.pick(size=40*number_of_branches,homogeneous=True)
+                self.networks[AN][ATr].rng_points = self.networks[AN][ATr].rng_points.tolist()
+                # new
+                self.networks[AN][ATr].rng_points,_ = self.boundary.pick(size=len(self.boundary.tet_verts),homogeneous=True,replacement=False)
+                self.networks[AN][ATr].rng_points = self.networks[AN][ATr].rng_points.tolist()
+                for n in tqdm(range(len(self.networks[AN][ATr].rng_points))):
+                    pt = np.array(self.networks[AN][ATr].rng_points.pop(0))
+                    if exact:
+                        other_vessels,distances = close_exact(self.networks[AN][ATr].data,pt)
+                    else:
+                        other_vessels,distances = close(self.networks[AN][ATr].data,pt)
+                    minimum_distance = min(distances)
+                    #if minimum_distance < 4*forest.networks[nid][njd].data[other_vessels[0],21]:
+                    #    continue
+                    retry = False
+                    for idx in active_networks:
+                        for jdx in list(range(self.trees_per_network[idx])):
+                            if idx == AN:
+                                continue
+                            if exact:
+                                other_vessels,distances = close_exact(self.networks[idx][jdx].data,pt)
                             else:
-                                check_trees = list(range(len(self.networks[nikd])))
-                            for njkd in check_trees:
-                                if not self.networks[nikd][njkd].collision_free(new_vessels):
-                                    repeat = True
+                                other_vessels,distances = close(self.networks[idx][jdx].data,pt)
+                            if min(distances) < minimum_distance:
+                                retry = True
+                                break
+                        if retry:
+                            break
+                    if retry:
+                        continue
+                    else:
+                        self.networks[AN][ATr].rng_points.insert(-1,pt.tolist())
+                #new
+        if not self.compete:
+            while len(active_networks) > 0:
+                for nid in active_networks:
+                    number_of_trees = len(self.networks[nid])
+                    for njd in range(number_of_trees):
+                        success = False
+                        while not success:
+                            vessel,data,sub_division_map,sub_division_index = self.networks[nid][njd].add(-1,0,isforest=True,radius_buffer=radius_buffer)
+                            new_vessels = np.vstack((data[vessel,:],data[-2,:],data[-1,:]))
+                            repeat = False
+                            for nikd in range(len(self.networks)):
+                                if nikd == nid:
+                                    check_trees = list(range(len(self.networks[nikd])))
+                                    check_trees.remove(njd)
+                                else:
+                                    check_trees = list(range(len(self.networks[nikd])))
+                                for njkd in check_trees:
+                                    if not self.networks[nikd][njkd].collision_free(new_vessels,radius_buffer=radius_buffer):
+                                        repeat = True
+                                        break
+                                if repeat:
                                     break
                             if repeat:
-                                break
-                        if repeat:
-                            continue
-                        else:
-                            success = True
-                    self.networks[nid][njd].data = data
-                    self.networks[nid][njd].parameters['edge_num'] += 2
-                    self.networks[nid][njd].sub_division_map = sub_division_map
-                    self.networks[nid][njd].sub_division_index = sub_division_index
-                print(self.networks[nid][0].parameters['edge_num'])
-                if self.networks[nid][0].parameters['edge_num'] >= exit_number[nid]:
-                    active_networks.remove(nid)
+                                continue
+                            else:
+                                success = True
+                        self.networks[nid][njd].data = data
+                        self.networks[nid][njd].parameters['edge_num'] += 2
+                        self.networks[nid][njd].sub_division_map = sub_division_map
+                        self.networks[nid][njd].sub_division_index = sub_division_index
+                    #print(self.networks[nid][0].parameters['edge_num'])
+                    if self.networks[nid][0].parameters['edge_num'] >= exit_number[nid]:
+                        active_networks.remove(nid)
+        else:
+            for i in tqdm(range(number_of_branches),desc="Adding branches"):
+                compete_add(self,network_ids=network_id,radius_buffer=radius_buffer)
 
-    def export(self,show=True,resolution=100,final=False):
+    def show(self,show=True,resolution=100,final=False,merged_trees=False):
+        if merged_trees:
+            colors = ['r','b','g','y']
+            plotter = pv.Plotter()
+            merged_list = []
+            for i in range(self.number_of_networks):
+                for j in range(self.trees_per_network[i]):
+                    merged_list.append(self.networks[i][j].export(cylinders=True))
+                    plotter.add_mesh(merged_list[-1],colors[i])
+            return plotter,merged_list
         model_networks = []
-        for network in self.networks:
+        for net_id,network in enumerate(self.networks):
             model_trees = []
-            for network_tree in network:
+            for tree_id,network_tree in enumerate(network):
                 model = []
                 for edge in range(network_tree.parameters['edge_num']):
-                    center = tuple((network_tree.data[edge,0:3] + network_tree.data[edge,3:6])/2)
+                    if network_tree.data[edge,15]==-1 and network_tree.data[edge,16]==-1:
+                        dis_point = (network_tree.data[edge,0:3] + network_tree.data[edge,3:6])/2
+                        vessel_length = network_tree.data[edge,20]/2
+                    else:
+                        dis_point = network_tree.data[edge,3:6]
+                        vessel_length = network_tree.data[edge,20]
+                    center = tuple((network_tree.data[edge,0:3] + dis_point)/2)
                     radius = network_tree.data[edge,21]
                     direction = tuple(network_tree.data[edge,12:15])
-                    vessel_length = network_tree.data[edge,20]
+                    #vessel_length = network_tree.data[edge,20]
                     model.append(pv.Cylinder(radius=radius,height=vessel_length,
                                              center=center,direction=direction,
                                              resolution=resolution))
+                if not np.any(self.connections[net_id] is None):
+                    for edge in range(self.connections[net_id][tree_id].shape[0]):
+                        term = self.connections[net_id][tree_id][edge,:]
+                        center = tuple((term[0:3]+term[3:6])/2)
+                        radius = term[21]
+                        direction = term[12:15]
+                        vessel_length = term[20]
+                        model.append(pv.Cylinder(radius=radius,height=vessel_length,
+                                                 center=center,direction=direction,
+                                                 resolution=resolution))
+                    #for edge in range(len(self.connections[net_id][0])):
+                    #    term = network_tree.data[self.assignments[net_id][tree_id][edge],:]
+                    #    conn = self.connections[net_id][0][edge]
+                    #    center = tuple((term[3:6] + conn)/2)
+                    #    radius = term[21]
+                    #    direction = tuple(conn-term[3:6])
+                    #    vessel_length = np.linalg.norm(conn-term[3:6])
+                    #    model.append(pv.Cylinder(radius=radius,height=vessel_length,
+                    #                             center=center,direction=direction,
+                    #                             resolution=resolution))
                 model_trees.append(model)
             model_networks.append(model_trees)
         """
@@ -446,7 +568,7 @@ class forest:
             plot = pv.Plotter()
             colors = ['r','b','g','y']
             #plot.set_background(color=[253,250,219])
-            for model_network in model_networks:
+            for model_idx,model_network in enumerate(model_networks):
                 for color_idx, model_tree in enumerate(model_network):
                     for model in model_tree:
                         plot.add_mesh(model,colors[color_idx])
@@ -463,6 +585,104 @@ class forest:
             #plot.close()
             return plot
 
+    def connect(self,network_id=-1,buffer=None,curve_sample_size_min=5,curve_sample_size_max=11,curve_degree=2):
+        self.connections,self.assignments = connect(self,network_id=network_id,buffer=buffer)
+        self.connections = smooth(self,curve_sample_size_min=curve_sample_size_min,curve_sample_size_max=curve_sample_size_max,curve_degree=curve_degree)
+
+    def export(self,steady=True,apply_distal_resistance=False,gui=False):
+        ALL_POINTS  = []
+        ALL_RADII   = []
+        ALL_NORMALS = []
+        CONNECTED_COPY = []
+        for network in range(self.number_of_networks):
+            network_points  = []
+            network_radii   = []
+            network_normals = []
+            network_endpoints = []
+            network_copy = []
+            for tree in range(self.trees_per_network[network]):
+                tree_copy = deepcopy(self.networks[network][tree].data)
+                tree_copy_terminals = tree_copy[np.argwhere(tree_copy[:,15]==-1).flatten(),:]
+                tree_copy_terminals = tree_copy_terminals[np.argwhere(tree_copy_terminals[:,16]==-1).flatten(),-1].astype(int)
+                tree_copy[tree_copy_terminals,3:6] = (tree_copy[tree_copy_terminals,0:3] + tree_copy[tree_copy_terminals,3:6])/2
+                tree_copy[tree_copy_terminals,20] = tree_copy[tree_copy_terminals,20]/2
+                max_vessel_id = max(tree_copy[:,-1])
+                conn_parents = np.argwhere(self.connections[network][tree][:,17] <= max_vessel_id).flatten()
+                tree_copy[self.connections[network][tree][conn_parents,17].astype(int),15] = self.connections[network][tree][conn_parents,-1]
+
+                tree_copy = np.vstack((tree_copy,self.connections[network][tree]))
+                network_copy.append(tree_copy)
+                interp_xyz,interp_r,interp_n,frames,branches = get_interpolated_sv_data(tree_copy)
+                points,radii,normals = sv_data(interp_xyz,interp_r,radius_buffer=0.01)
+                network_points.append(points)
+                network_radii.append(radii)
+                network_normals.append(normals)
+                pts = [p[-1] for p in points]
+                network_endpoints.append(pts)
+            CONNECTED_COPY.append(network_copy)
+            # Calculate Vessel Reordering relative to the first vessel of the network
+            order = [list(range(len(network_endpoints[0])))]
+            for tree in range(1,self.trees_per_network[network]):
+                tmp = []
+                for idx in order[0]:
+                    #tmp.append(np.argwhere(np.all(np.isclose(e[0][0],network_endpoints[]),axis=1)).flatten()[0])
+                    tmp.append(network_endpoints[tree].index(network_endpoints[0][idx]))
+                order.append(tmp)
+            # Reorder Vessels
+            reordered_points  = []
+            reordered_radii   = []
+            reordered_normals = []
+            for tree in range(self.trees_per_network[network]):
+                tmp_points  = []
+                tmp_radii   = []
+                tmp_normals = []
+                for idx in order[tree]:
+                    tmp_points.append(network_points[tree][idx])
+                    tmp_radii.append(network_radii[tree][idx])
+                    tmp_normals.append(network_normals[tree][idx])
+                reordered_points.append(tmp_points)
+                reordered_radii.append(tmp_radii)
+                reordered_normals.append(tmp_normals)
+                print('reorder: {}'.format(len(reordered_points[-1])))
+            # Reverse and Combine
+            final_points  = [reordered_points[0]]
+            final_radii   = [reordered_radii[0]]
+            final_normals = [reordered_normals[0]]
+            for tree in range(1,self.trees_per_network[network]):
+                if tree == 1:
+                    for vessel in range(len(reordered_points[tree])):
+                        final_points[0][vessel].extend(list(reversed(reordered_points[tree][vessel]))[1:])
+                        final_radii[0][vessel].extend(list(reversed(reordered_radii[tree][vessel]))[1:])
+                        final_normals[0][vessel].extend(list(reversed(reordered_normals[tree][vessel]))[1:])
+                else:
+                    tmp_points  = []
+                    tmp_radii   = []
+                    tmp_normals = []
+                    for vessel in range(len(reordered_points[tree])):
+                        tmp_points.append(list(reversed(reordered_points[tree][vessel])))
+                        tmp_radii.append(list(reversed(reordered_radii[tree][vessel])))
+                        tmp_normals.append(list(reversed(reordered_normals[tree][vessel])))
+                    final_points.append(tmp_points)
+                    final_radii.append(tmp_radii)
+                    final_normals.append(tmp_normals)
+            # Export Components of Network to SV Builder for Code Generation
+            for component in range(len(final_points)):
+                if steady:
+                    time = [0, 1]
+                    flow = [self.networks[network][0].data[0,22], self.networks[network][0].data[0,22]]
+                else:
+                    time,flow = generate_physiologic_wave(self.networks[network][0].data[0,22],self.networks[network][0].data[0,21]*2)
+                    time = time.tolist()
+                    flow = flow.tolist()
+                    flow[-1] = flow[0]
+                if apply_distal_resistance:
+                    R = self.networks[network][0].parameters['Pterm']/self.networks[network][0].data[0,22]
+                else:
+                    R = 0
+                options = file_options(time=time,flow=flow,gui=gui,distal_resistance=R)
+                build(final_points[component],final_radii[component],final_normals[component],options)
+        return final_points,final_radii,final_normals,CONNECTED_COPY
+
 def perfusion_territory(tree,subdivisions=0,mesh_file=None,tree_file=None):
     terminals = tree.data[tree.data[:,15]<0,:]
     terminals = terminals[terminals[:,16]<0,:]
